'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CALL_API = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

require('whatwg-fetch');

var _humps = require('humps');

var _isEmpty = require('lodash/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getLinkRel = function getLinkRel(response, relVal) {
  var link = response.headers.get('link');

  if (!link) {
    return null;
  }

  var rel = link.split(',').find(function (s) {
    return s.indexOf('rel="' + relVal + '"') > -1;
  });

  if (!rel) {
    return null;
  }

  return rel.split(';')[0].slice(1, -1);
};

var callApi = function callApi(url, options) {
  return fetch(url, options).then(function (response) {
    return response.json().then(function (json) {
      if (!response.ok) {
        return Promise.reject(_extends({}, json, { status: response.status }));
      }

      var camelizedJson = (0, _humps.camelizeKeys)(json);
      var next = getLinkRel(response, 'next');
      var prev = getLinkRel(response, 'prev');

      return _extends({
        data: camelizedJson
      }, next && { next: next }, prev && { prev: prev });
    });
  });
};

// Action key that carries API call info interpreted by this Redux middleware.
var CALL_API = exports.CALL_API = 'CALL API';

// A Redux middleware that interprets actions with CALL_API info specified.
// Performs the call and promises when such actions are dispatched.

exports.default = function (store) {
  return function (next) {
    return function (action) {
      var callAPI = action[CALL_API];

      if (typeof callAPI === 'undefined') {
        return next(action);
      }

      var endpoint = callAPI.endpoint,
          query = callAPI.query,
          body = callAPI.body,
          method = callAPI.method,
          token = callAPI.token,
          types = callAPI.types;


      if (typeof endpoint !== 'string') {
        throw new Error('Specify a string endpoint URL.');
      }
      if (!Array.isArray(types) || types.length !== 3) {
        throw new Error('Expected an array of three action types.');
      }
      if (!types.every(function (type) {
        return typeof type === 'string';
      })) {
        throw new Error('Expected action types to be strings.');
      }
      if (query && body) {
        throw new Error('Do not call with both query and body');
      }
      if (!method) {
        throw new Error('Specify a method for api call.');
      }
      if (['GET', 'POST', 'PUT', 'DELETE'].indexOf(method) === -1) {
        throw new Error('Method must be one of {GET, POST, PUT, DELETE}');
      }

      var url = _constants.API_ROOT + '/' + endpoint;

      if (query && !(0, _isEmpty2.default)(query)) {
        var qs = Object.keys(query).map(function (key) {
          var data = query[key];
          if (Array.isArray(data) && data.length > 0) {
            return data.map(function (val) {
              return key + '=' + encodeURIComponent(val);
            }).join('&');
          } else {
            return key + '=' + encodeURIComponent(data);
          }
        }).join('&');

        url = url + '?' + qs;
      }

      var options = {
        credentials: 'include',
        method: method
      };

      if (token) {
        options.headers.Authorization = token;
      }

      if (body) {
        if (body instanceof FormData) {
          options.body = body;
        } else {
          options.body = JSON.stringify(body);
          options.headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
          };
        }
      }

      var actionWith = function actionWith(data) {
        var finalAction = _extends({}, action, data);
        delete finalAction[CALL_API];
        return finalAction;
      };

      var _types = _slicedToArray(types, 3),
          requestType = _types[0],
          successType = _types[1],
          failureType = _types[2];

      next(actionWith({ type: requestType }));

      return callApi(url, options).then(function (response) {
        return next(actionWith({
          response: response,
          type: successType
        }));
      }, function (error) {
        return next(actionWith({
          error: error,
          type: failureType
        }));
      });
    };
  };
};